// lcd.c
// LCD functions for SED1520 Controller (122x32 Pixel)
// Autor: Bertram Schall
// Date: 31.01.07
// Last Revision: 31.01.07

#include <msp430x14x.h>
#include "lcd.h"
#include "font.h"
#include "string.h"

/***************************************************/
/*   High Level Functions                          */
/***************************************************/
void print_str (unsigned char page, unsigned char column, char *s, unsigned char mod, unsigned char embedd){
  unsigned char i;
  for(i=0;i<strlen(s);i++){
    column=print_char(page,column,s[i],mod,embedd);
  }
}

unsigned char print_char (unsigned char page, unsigned char column, char c, unsigned char mod, unsigned char embedd){
  unsigned char i, ch, tmp, mask;
  //Richtige Reihe
  lcd_selectside(3);
  lcd_setpage(page);
  for(i=0;i<fontsize[c-32];i++){
    //Richtige Seite & Column
    if((column+i)<61){
      lcd_selectside(1);
      lcd_setcolumn(column+i);
    }
    else{
      lcd_selectside(2);
      lcd_setcolumn((column-61)+i);
    }
    ch=font[((c-32)*3)+i];
    //Shift
    if(mod>0)
      ch=ch>>mod;
    if(embedd==1){
      //mask reset
      mask=0b11111110;
      mask=mask>>(mod-1);
      mask=~mask;
      lcd_startrmw();
      lcd_readdata();
      tmp=lcd_readdata();
      tmp&=mask;
      tmp|=ch;
      lcd_writedata(tmp);
    }
    else
      lcd_writedata(ch);
  }
  //Abstand... 1 Column
  if(embedd==0)
    lcd_writedata(0x00);
  else{
      lcd_readdata();
      tmp=lcd_readdata();
      tmp&=mask;
      lcd_writedata(tmp);
      lcd_endrmw();
  }
  return (column+fontsize[c-32]+1);
}

void lcd_invert (unsigned char line, unsigned char from, unsigned char to){
  unsigned char tmp,i;
  //Berechnen der Page
  lcd_selectside(3);
  lcd_setpage(line);
  //Column Berechnen + Mod
  for(i=0;i<(to-from);i++){
    //clac Column + Side
    if((from+i)<61){
      lcd_selectside(1);
      lcd_setcolumn(from+i);
    }
    else{
      lcd_selectside(2);
      lcd_setcolumn(from+i-61);
    }
    //RMW
    //Dummy Read
    lcd_startrmw();
    lcd_readdata();
    tmp=lcd_readdata();
    tmp=~tmp;
    lcd_writedata(tmp);
    lcd_endrmw();
  }
}

void lcd_drawwindow (unsigned char width){
  unsigned char i,j,from,to,data;
  from=61-(width/2);
  to=61+(width/2);
  for(j=1;j<3;j++){
    //Zeile auswhlen
    lcd_selectside(3);
    lcd_setpage(j);
    for(i=from;i<=to;i++){
      //clac Column + Side
      if(i<61){
        lcd_selectside(1);
        lcd_setcolumn(i);
      }
      else{
        lcd_selectside(2);
        lcd_setcolumn(i-61);
      }
      //erste Zeile
      if(j==1){
        data=0x01;
        if(i==from)
          data=0xFF;
        if(i==to)
          data=0xFF;
      }
      //zweite Zeile
      if(j==2){
        data=0x80;
        if(i==from)
          data=0xFF;
        if(i==to)
          data=0xFF;
      }
      lcd_writedata(data);
    }
  }
}

void lcd_bmp(char* bmp){
  int i,j;
  for(i=0;i<4;i++){ //Zeilen
    //Left side
    lcd_selectside(3);
    lcd_setcolumn(0);
    lcd_setpage(i);
    lcd_selectside(1);
    for(j=0;j<122;j++){
      lcd_writedata(bmp[i*122+j]);
      if(j==60)
        lcd_selectside(2);
    }
  }
}

void lcd_line (char x1, char y1, char x2, char y2){
  unsigned char i;
  for(i=0;i<(x2-x1);i++){
    lcd_pixel(x1+i,((float)(y2-y1)/(x2-x1))*i+x1);
  }
}

void lcd_pixel (unsigned char x, unsigned char y){
  unsigned char tmp;
  //Berechnen der Page
  lcd_selectside(3);
  lcd_setpage((unsigned char)(y/8));
  //Berechnen der Column
  if(x<61){
    lcd_selectside(1);
    lcd_setcolumn(x);
  }
  else{
    lcd_selectside(2);
    lcd_setcolumn(x-61);
  }
  //RMW
  lcd_startrmw();
  // Dummy read
  lcd_readdata();
  tmp=lcd_readdata();
  tmp|=(0x01<<(y%8));
  lcd_writedata(tmp);
  lcd_endrmw();
}

/***************************************************/
/*   Low Level Functions                           */
/***************************************************/
void lcd_init(void){
  P4DIR=0xFF; //default->Output
  P4OUT=0x00; //default
  P5DIR=0xFF; //default->Output
  P5OUT=0x10;
  //68 CPU
  RESL;
  _NOP();
  RESH;
  E1L;
  E2L;
  lcd_selectside(3);
  lcd_onoff(0xFF);//on
  lcd_static(0x00);//normal operation
  lcd_setcolumn(0x00);//0
  lcd_setpage(0x00);//0
  lcd_duty(0xFF);//1/32
  lcd_selectADC(0x00);//CW Output
  lcd_endrmw();//No RMW
  lcd_blank();
  lcd_selectside(0);
}

void lcd_blank(void){
  int i,j;
  lcd_selectside(3);
  for(i=0;i<4;i++){
    lcd_setpage(i);
    lcd_setcolumn(0x00);
    for(j=0;j<61;j++){
      lcd_writedata(0x00);
    }
  }
}

/***************************************************/
/*   HAL                                           */
/***************************************************/
static unsigned char dside;

void lcd_burst(void){
  switch(dside){
    case 0:break;
    case 1:E1H;E1L;break;
    case 2:E2H;E2L;break;
    case 3:E1H;E2H;E1L;E2L;break;
    default:;
  }  
}

void lcd_selectside(unsigned int side){
  dside=side;
}

void lcd_onoff(unsigned char status){
  A0L;
  DSOUT;
  P4OUT=(0xAE)|(status>>7);
  RWL;
  lcd_burst();
}

void lcd_startline(unsigned char line){
  A0L;
  DSOUT;
  P4OUT=(0xC0)|line;
  RWL;
  lcd_burst();
}

void lcd_setpage(unsigned char page){
  A0L;
  DSOUT;
  P4OUT=(0xB8)|page;
  RWL;
  lcd_burst();
}

void lcd_setcolumn(unsigned char column){
  A0L;
  DSOUT;
  P4OUT=(0x00)|column;
  RWL;
  lcd_burst();
}

unsigned char lcd_status(void){
  unsigned char status;
  A0L;
  DSIN;
  RWH;
  lcd_burst();
  status=P4IN;
  return status;
}

void lcd_writedata(unsigned char data){  //Fr hohen Takt BSY abfragen
  A0H;
  DSOUT;
  P4OUT=data;
  RWL;
  lcd_burst();
}

unsigned char lcd_readdata(void){ //Fr hohen Takt BSY abfragen
  unsigned char data;
  A0H;
  DSIN;
  RWH;
  lcd_burst();
  data=P4IN;
  return data;
}

void lcd_selectADC(unsigned char adc){
  A0L;
  DSOUT;
  P4OUT=(0xA0)|(adc>>7);
  RWL;
  lcd_burst();
}

void lcd_static(unsigned char stat){
  A0L;
  DSOUT;
  P4OUT=(0xA4)|(stat>>7);
  RWL;
  lcd_burst();
}

void lcd_duty (unsigned char duty){
  A0L;
  DSOUT;
  P4OUT=(0xA8)|(duty>>7);
  RWL;
  lcd_burst();
}

void lcd_startrmw(void){
  A0L;
  DSOUT;
  P4OUT=0xE0;
  RWL;
  lcd_burst();
}

void lcd_endrmw(void){
  A0L;
  DSOUT;
  P4OUT=0xEE;
  RWL;
  lcd_burst();
}

void lcd_softres(void){
  A0L;
  DSOUT;
  P4OUT=0xE2;
  RWL;
  lcd_burst();
}

void lcd_bl(unsigned char bl){
  if(bl==0xFF)
    BLH;
  else
    BLL;
}
